import requests
import numpy as np
import time
import math
import os
import glob
import tqdm

import random


import img_correction


class CameraRequest:
    def __init__(self, ip, verbose=False):
        self.ip = ip
        self.fmt_status = "/status"
        self.fmt_move = "/move?az={:.2f}&el={:.2f}"
        self.fmt_shoot = "/shoot?led_on={}"
        self.verbose = verbose

    def get_txt(self, page, verbose=False):
        url = "http://" + self.ip + page
        r = requests.get(url)
        txt = r.text
        if verbose:
            print("GET:", url)
        return txt

    def wait(self):
        txt = "BUSY"
        page = self.fmt_status
        while txt != "IDLE":
            time.sleep(0.25)
            txt = self.get_txt(page)

    def move(self, az, el):
        page = self.fmt_move.format(az, el)
        txt = self.get_txt(page, self.verbose)
        if txt != "OK":
            return False
        self.wait()
        return True

    def shoot(self, filename, led_on=True):
        page = self.fmt_shoot.format(int(led_on))
        url = "http://" + self.ip + page
        r = requests.get(url, allow_redirects=True)
        if self.verbose:
            print("GET:", url)
        open(filename, 'wb').write(r.content)


def acquire_list(az_list, el_list, name_list, dir_yuv, dir_rgb, ip, output_ext="jpg"):
    cam = CameraRequest(ip, True)

    rgb_path_list = []
    
    os.makedirs(dir_yuv, exist_ok=True)
    os.makedirs(dir_rgb, exist_ok=True)

    for az, el, img_name in zip(az_list, el_list, name_list):
        yuv_path = os.path.join(dir_yuv, img_name+".yuv")
        rgb_path = os.path.join(dir_rgb, img_name+"."+output_ext)
        cam.move(az, el)
        cam.shoot(yuv_path)
        img_correction.convert_yuv(yuv_path, rgb_path)
        rgb_path_list.append(rgb_path)

    if output_ext == "jpg":
        img_correction.set_tags(*rgb_path_list)


def main():
    ip = r"INSERT_IP_HERE"
    dir_yuv = r"data"
    dir_rgb = r"data\jpg"
    N_az = 10
    N_el = 4
    , dir_rgb, ip

    filename_fmt = "img_{}_{}"

    az_1d = np.arange(N_az) * 360/N_az
    el_1d = np.linspace(6, 43, N_el)

    az_list, el_list = np.meshgrid(az_1d, el_1d, indexing="ij")
    az_list = az_list.flatten()
    el_list = el_list.flatten()

    filename_list = []

    for i in range(N_az):
        for j in range(N_el):
            filename_list.append(filename_fmt.format(i, j))

    acquire_list(az_list, el_list, filename_list,
                 dir_yuv,
                 dir_rgb,
                 ip)



if __name__ == "__main__":
    main()
    # main_repeat()
    # main_circle()
